@@ -16,9 +16,11 @@ import (
 	"strings"
 	"time"
 
+	"github.com/yorukot/superfile/src/internal/utils"
+
+	"github.com/yorukot/superfile/src/internal/common"
+
 	"github.com/lithammer/shortuuid"
-	"github.com/reinhrst/fzf-lib"
-	"github.com/yorukot/superfile/src/config/icon"
 )
 
 // Check if the directory is external disk path
@@ -32,7 +34,7 @@ func isExternalDiskPath(path string) bool {
 	// This is very vague. You cannot tell if a path is belonging to an external partition
 	// if you dont define the source path to compare with
 	// But making this true will cause slow file operations based on current implementation
-	if runtime.GOOS == "windows" {
+	if runtime.GOOS == utils.OsWindows {
 		return false
 	}
 
@@ -48,63 +50,6 @@ func isExternalDiskPath(path string) bool {
 		strings.HasPrefix(path, "/Volumes")
 }
 
-func shouldListDisk(mountPoint string) bool {
-	if runtime.GOOS == "windows" {
-		// We need to get C:, D: drive etc in the list
-		return true
-	}
-
-	// Should always list the main disk
-	if mountPoint == "/" {
-		return true
-	}
-
-	// Todo : make a configurable field in config.yaml
-	// excluded_disk_mounts = ["/Volumes/.timemachine"]
-	// Mountpoints that are in subdirectory of disk_mounts
-	// but still are to be excluded in disk section of sidebar
-	if strings.HasPrefix(mountPoint, "/Volumes/.timemachine") {
-		return false
-	}
-
-	// We avoid listing all mounted partitions (Otherwise listed disk could get huge)
-	// but only a few partitions that usually corresponds to external physical devices
-	// For example : mounts like /boot, /var/ will get skipped
-	// This can be inaccurate based on your system setup if you mount any external devices
-	// on other directories, or if you have some extra mounts on these directories
-	// Todo : make a configurable field in config.yaml
-	// disk_mounts = ["/mnt", "/media", "/run/media", "/Volumes"]
-	// Only block devicies that are mounted on these or any subdirectory of these Mountpoints
-	// Will be shown in disk sidebar
-	return strings.HasPrefix(mountPoint, "/mnt") ||
-		strings.HasPrefix(mountPoint, "/media") ||
-		strings.HasPrefix(mountPoint, "/run/media") ||
-		strings.HasPrefix(mountPoint, "/Volumes")
-}
-
-func diskName(mountPoint string) string {
-	// In windows we dont want to use filepath.Base as it returns "\" for when
-	// mountPoint is any drive root "C:", "D:", etc. Hence causing same name
-	// for each drive
-	if runtime.GOOS == "windows" {
-		return mountPoint
-	}
-
-	// This might cause duplicate names in case you mount two devices in
-	// /mnt/usb and /mnt/dir2/usb . Full mountpoint is a more accurate way
-	// but that results in messy UI, hence we do this.
-	return filepath.Base(mountPoint)
-}
-
-func diskLocation(mountPoint string) string {
-	// In windows if you are in "C:\some\path", "cd C:" will not cd to root of C: drive
-	// but "cd C:\" will
-	if runtime.GOOS == "windows" {
-		return filepath.Join(mountPoint, "\\")
-	}
-	return mountPoint
-}
-
 func returnFocusType(focusPanel focusPanelType) filePanelFocusType {
 	if focusPanel == nonePanelFocus {
 		return focus
@@ -112,11 +57,11 @@ func returnFocusType(focusPanel focusPanelType) filePanelFocusType {
 	return secondFocus
 }
 
-func returnDirElement(location string, displayDotFile bool, sortOptions sortOptionsModelData) (directoryElement []element) {
+func returnDirElement(location string, displayDotFile bool, sortOptions sortOptionsModelData) []element {
 	dirEntries, err := os.ReadDir(location)
 	if err != nil {
 		slog.Error("Error while return folder element function", "error", err)
-		return directoryElement
+		return nil
 	}
 
 	dirEntries = slices.DeleteFunc(dirEntries, func(e os.DirEntry) bool {
@@ -127,7 +72,7 @@ func returnDirElement(location string, displayDotFile bool, sortOptions sortOpti
 
 	// No files/directoes to process
 	if len(dirEntries) == 0 {
-		return directoryElement
+		return nil
 	}
 
 	// Sort files
@@ -142,11 +87,10 @@ func returnDirElement(location string, displayDotFile bool, sortOptions sortOpti
 			if dirEntries[i].IsDir() != dirEntries[j].IsDir() {
 				return dirEntries[i].IsDir()
 			}
-			if Config.CaseSensitiveSort {
+			if common.Config.CaseSensitiveSort {
 				return dirEntries[i].Name() < dirEntries[j].Name() != reversed
-			} else {
-				return strings.ToLower(dirEntries[i].Name()) < strings.ToLower(dirEntries[j].Name()) != reversed
 			}
+			return strings.ToLower(dirEntries[i].Name()) < strings.ToLower(dirEntries[j].Name()) != reversed
 		}
 	case "Size":
 		order = func(i, j int) bool {
@@ -172,13 +116,11 @@ func returnDirElement(location string, displayDotFile bool, sortOptions sortOpti
 					slog.Error("Error when reading directory during sort", "error", err)
 				}
 				return len(filesI) < len(filesJ) != reversed
-			} else {
-				// No need for err check, we already filtered out dirEntries with err != nil in Info() call
-				fileInfoI, _ := dirEntries[i].Info()
-				fileInfoJ, _ := dirEntries[j].Info()
-				return fileInfoI.Size() < fileInfoJ.Size() != reversed
 			}
-
+			// No need for err check, we already filtered out dirEntries with err != nil in Info() call
+			fileInfoI, _ := dirEntries[i].Info()
+			fileInfoJ, _ := dirEntries[j].Info()
+			return fileInfoI.Size() < fileInfoJ.Size() != reversed
 		}
 	case "Date Modified":
 		order = func(i, j int) bool {
@@ -190,6 +132,8 @@ func returnDirElement(location string, displayDotFile bool, sortOptions sortOpti
 	}
 
 	sort.Slice(dirEntries, order)
+	// Preallocate for efficiency
+	directoryElement := make([]element, 0, len(dirEntries))
 	for _, item := range dirEntries {
 		directoryElement = append(directoryElement, element{
 			name:      item.Name(),
@@ -200,14 +144,17 @@ func returnDirElement(location string, displayDotFile bool, sortOptions sortOpti
 	return directoryElement
 }
 
-func returnDirElementBySearchString(location string, displayDotFile bool, searchString string) (dirElement []element) {
-
+func returnDirElementBySearchString(location string, displayDotFile bool, searchString string) []element {
 	items, err := os.ReadDir(location)
 	if err != nil {
 		slog.Error("Error while return folder element function", "error", err)
 		return []element{}
 	}
 
+	if len(items) == 0 {
+		return []element{}
+	}
+
 	folderElementMap := map[string]element{}
 	fileAndDirectories := []string{}
 
@@ -231,7 +178,9 @@ func returnDirElementBySearchString(location string, displayDotFile bool, search
 	}
 	// https://github.com/reinhrst/fzf-lib/blob/main/core.go#L43
 	// No sorting needed. fzf.DefaultOptions() already return values ordered on Score
-	for _, item := range fzfSearch(searchString, fileAndDirectories) {
+	fzfResults := utils.FzfSearch(searchString, fileAndDirectories)
+	dirElement := make([]element, 0, len(fzfResults))
+	for _, item := range fzfResults {
 		resultItem := folderElementMap[item.Key]
 		dirElement = append(dirElement, resultItem)
 	}
@@ -239,15 +188,6 @@ func returnDirElementBySearchString(location string, displayDotFile bool, search
 	return dirElement
 }
 
-// Returning a string slice causes inefficiency in current usage
-func fzfSearch(query string, source []string) []fzf.MatchResult {
-	fzfSearcher := fzf.New(source, fzf.DefaultOptions())
-	fzfSearcher.Search(query)
-	fzfResults := <-fzfSearcher.GetResultChannel()
-	fzfSearcher.End()
-	return fzfResults.Matches
-}
-
 func panelElementHeight(mainPanelHeight int) int {
 	return mainPanelHeight - 3
 }
@@ -262,12 +202,6 @@ func arrayContains(s []string, str string) bool {
 	return false
 }
 
-// Todo : Eventually we want to remove all such usage that can result in app exiting abruptly
-func LogAndExit(msg string, values ...any) {
-	slog.Error(msg, values...)
-	os.Exit(1)
-}
-
 func removeElementByValue(slice []string, value string) []string {
 	newSlice := []string{}
 	for _, v := range slice {
@@ -341,7 +275,7 @@ func (m *model) returnMetaData() {
 	id := shortuuid.New()
 
 	message := channelMessage{
-		messageId:   id,
+		messageID:   id,
 		messageType: sendMetadata,
 		metadata:    m.fileMetaData.metaData,
 	}
@@ -372,8 +306,8 @@ func (m *model) returnMetaData() {
 	fileInfo, err := os.Stat(filePath)
 
 	if isSymlink(filePath) {
-		_, err := filepath.EvalSymlinks(filePath)
-		if err != nil {
+		_, symlinkErr := filepath.EvalSymlinks(filePath)
+		if symlinkErr != nil {
 			m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"Link file is broken!", ""})
 		} else {
 			m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"This is a link file.", ""})
@@ -381,7 +315,6 @@ func (m *model) returnMetaData() {
 		message.metadata = m.fileMetaData.metaData
 		channel <- message
 		return
-
 	}
 
 	if err != nil {
@@ -389,12 +322,14 @@ func (m *model) returnMetaData() {
 	}
 
 	if fileInfo.IsDir() {
-		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"FolderName", fileInfo.Name()})
+		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"Name", fileInfo.Name()})
 		if m.focusPanel == metadataFocus {
-			m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"FolderSize", formatFileSize(dirSize(filePath))})
+			// Todo : Calling dirSize() could be expensive for large directories, as it recursively
+			// walks the entire tree. Consider lazy loading, caching, or an async approach to avoid UI lockups.
+			m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"Size", common.FormatFileSize(dirSize(filePath))})
 		}
-		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"FolderModifyDate", fileInfo.ModTime().String()})
-		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"FolderPermissions", fileInfo.Mode().String()})
+		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"Date Modified", fileInfo.ModTime().String()})
+		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"Permissions", fileInfo.Mode().String()})
 		message.metadata = m.fileMetaData.metaData
 		channel <- message
 		return
@@ -406,8 +341,7 @@ func (m *model) returnMetaData() {
 		return
 	}
 
-	if Config.Metadata && checkIsSymlinked.Mode()&os.ModeSymlink == 0 && et != nil {
-
+	if common.Config.Metadata && checkIsSymlinked.Mode()&os.ModeSymlink == 0 && et != nil {
 		fileInfos := et.ExtractMetadata(filePath)
 
 		for _, fileInfo := range fileInfos {
@@ -422,12 +356,12 @@ func (m *model) returnMetaData() {
 			}
 		}
 	} else {
-		fileName := [2]string{"FileName", fileInfo.Name()}
-		fileSize := [2]string{"FileSize", formatFileSize(fileInfo.Size())}
-		fileModifyData := [2]string{"FileModifyDate", fileInfo.ModTime().String()}
-		filePermissions := [2]string{"FilePermissions", fileInfo.Mode().String()}
+		fileName := [2]string{"Name", fileInfo.Name()}
+		fileSize := [2]string{"Size", common.FormatFileSize(fileInfo.Size())}
+		fileModifyData := [2]string{"Date Modified", fileInfo.ModTime().String()}
+		filePermissions := [2]string{"Permissions", fileInfo.Mode().String()}
 
-		if Config.EnableMD5Checksum {
+		if common.Config.EnableMD5Checksum {
 			// Calculate MD5 checksum
 			checksum, err := calculateMD5Checksum(filePath)
 			if err != nil {
@@ -447,16 +381,22 @@ func (m *model) returnMetaData() {
 	panel.element[panel.cursor].metaData = m.fileMetaData.metaData
 }
 
+// Todo : Replace all usage of "m.fileModel.filePanels[m.filePanelFocusIndex]" with this
+// There are many usage
+func (m *model) getFocusedFilePanel() *filePanel {
+	return &m.fileModel.filePanels[m.filePanelFocusIndex]
+}
+
 func calculateMD5Checksum(filePath string) (string, error) {
 	file, err := os.Open(filePath)
 	if err != nil {
-		return "", fmt.Errorf("failed to open file: %v", err)
+		return "", fmt.Errorf("failed to open file: %w", err)
 	}
 	defer file.Close()
 
 	hash := md5.New()
 	if _, err := io.Copy(hash, file); err != nil {
-		return "", fmt.Errorf("failed to calculate MD5 checksum: %v", err)
+		return "", fmt.Errorf("failed to calculate MD5 checksum: %w", err)
 	}
 
 	checksum := hex.EncodeToString(hash.Sum(nil))
@@ -466,20 +406,21 @@ func calculateMD5Checksum(filePath string) (string, error) {
 // Get directory total size
 func dirSize(path string) int64 {
 	var size int64
-	err := filepath.WalkDir(path, func(_ string, entry os.DirEntry, err error) error {
+	// Its named walkErr to prevent shadowing
+	walkErr := filepath.WalkDir(path, func(_ string, entry os.DirEntry, err error) error {
 		if err != nil {
 			slog.Error("Dir size function error", "error", err)
 		}
 		if !entry.IsDir() {
-			info, err := entry.Info()
-			if err == nil {
+			info, infoErr := entry.Info()
+			if infoErr == nil {
 				size += info.Size()
 			}
 		}
 		return err
 	})
-	if err != nil {
-		slog.Error("errors during WalkDir", "error", err)
+	if walkErr != nil {
+		slog.Error("errors during WalkDir", "error", walkErr)
 	}
 	return size
 }
@@ -488,7 +429,7 @@ func dirSize(path string) int64 {
 func countFiles(dirPath string) (int, error) {
 	count := 0
 
-	err := filepath.Walk(dirPath, func(path string, info os.FileInfo, err error) error {
+	err := filepath.Walk(dirPath, func(_ string, info os.FileInfo, err error) error {
 		if err != nil {
 			return err
 		}
@@ -503,7 +444,6 @@ func countFiles(dirPath string) (int, error) {
 
 // Check whether is symlinks
 func isSymlink(filePath string) bool {
-
 	fileInfo, err := os.Lstat(filePath)
 	if err != nil {
 		return true
@@ -527,59 +467,3 @@ func isImageFile(filename string) bool {
 	ext := strings.ToLower(filepath.Ext(filename))
 	return imageExtensions[ext]
 }
-
-func getElementIcon(file string, IsDir bool) icon.IconStyle {
-	ext := strings.TrimPrefix(filepath.Ext(file), ".")
-	name := file
-
-	if !Config.Nerdfont {
-		return icon.IconStyle{
-			Icon:  "",
-			Color: theme.FilePanelFG,
-		}
-	}
-
-	if IsDir {
-		resultIcon := icon.Folders["folder"]
-		betterIcon, hasBetterIcon := icon.Folders[name]
-		if hasBetterIcon {
-			resultIcon = betterIcon
-		}
-		return resultIcon
-	} else {
-		// default icon for all files. try to find a better one though...
-		resultIcon := icon.Icons["file"]
-		// resolve aliased extensions
-		extKey := strings.ToLower(ext)
-		alias, hasAlias := icon.Aliases[extKey]
-		if hasAlias {
-			extKey = alias
-		}
-
-		// see if we can find a better icon based on extension alone
-		betterIcon, hasBetterIcon := icon.Icons[extKey]
-		if hasBetterIcon {
-			resultIcon = betterIcon
-		}
-
-		// now look for icons based on full names
-		fullName := name
-
-		fullName = strings.ToLower(fullName)
-		fullAlias, hasFullAlias := icon.Aliases[fullName]
-		if hasFullAlias {
-			fullName = fullAlias
-		}
-		bestIcon, hasBestIcon := icon.Icons[fullName]
-		if hasBestIcon {
-			resultIcon = bestIcon
-		}
-		if resultIcon.Color == "NONE" {
-			return icon.IconStyle{
-				Icon:  resultIcon.Icon,
-				Color: theme.FilePanelFG,
-			}
-		}
-		return resultIcon
-	}
-}
